using SecureShopBusinessLogic.BusinessLogics;
using SecureShopContracts.BusinessLogicsContracts;
using SecureShopContracts.StoragesContracts;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;
using SecureShopBusinessLogic.OfficePackage;
using SecureShopBusinessLogic.OfficePackage.Implements;
using SecureShopDatabaseImplement.Implements;
using SecureShopView;

namespace SecureShopView
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();
            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IFacilitiesStorage, FacilitiesStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IClientStorage, ClientStorage>();
            services.AddTransient<ISecureStorage, SecureStorage>();
            services.AddTransient<IFacilitiesLogic, FacilitiesLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<ISecureLogic, SecureLogic>();
            services.AddTransient<IClientLogic, ClientLogic>();
            services.AddTransient<IImplementerLogic, ImplementerLogic>();
            services.AddTransient<FormMain>();
            services.AddTransient<IImplementerStorage, ImplementerStorage>();
            services.AddTransient<IReportLogic, ReportLogic>();
            services.AddTransient<IWorkProcess, WorkModeling>();
            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();
            services.AddTransient<FormFacilitie>();
            services.AddTransient<FormReportSecureFacilities>();
            services.AddTransient<FormReportOrders>();
            services.AddTransient<FormImplementers>();
            services.AddTransient<FormImplementer>();
            services.AddTransient<FormFacilitiess>();
            services.AddTransient<FormCreateOrder>();
            services.AddTransient<FormSecure>();
            services.AddTransient<FormSecureFacilities>();
            services.AddTransient<FormSecures>();
            services.AddTransient<FormClients>();
        }

    }
}