﻿using SecureShopContracts.BindingModels;
using SecureShopContracts.ViewModels;
using SecureShopDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace SecureShopDatabaseImplement.Models
{
    public class Implementer : IImplementerModel
    {
        [Required]
        public string ImplementerFIO { get; private set; } = string.Empty;
        [Required]
        public string Password { get; private set; } = string.Empty;
        [Required]
        public int WorkExperience { get; private set; }
        [Required]
        public int Qualification { get; private set; }
        [Required]
        public int Id { get; private set; }
        [ForeignKey("ImplementerId")]
        public virtual List<Order> Orders { get; set; } = new();
        public static Implementer? Create(ImplementerBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Implementer()
            {
                Id = model.Id,
                ImplementerFIO = model.ImplementerFIO,
                Qualification = model.Qualification,
                WorkExperience = model.WorkExperience,
                Password = model.Password
            };
        }

        public static Implementer Create(ImplementerViewModel model)
        {
            return new Implementer
            {
                Id = model.Id,
                ImplementerFIO = model.ImplementerFIO,
                Qualification = model.Qualification,
                WorkExperience = model.WorkExperience,
                Password = model.Password
            };
        }

        public void Update(ImplementerBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ImplementerFIO = model.ImplementerFIO;
            Password = model.Password;
            Qualification = model.Qualification;
            WorkExperience = model.WorkExperience;
        }

        public ImplementerViewModel GetViewModel => new()
        {
            Id = Id,
            ImplementerFIO = ImplementerFIO,
            Qualification = Qualification,
            WorkExperience = WorkExperience,
            Password = Password
        };
    }
}